@@ -176,18 +176,20 @@ TF_TER	Feuchttemperatur
 GEWITTER	Gewitter
 JA_GEWITTER	Gewitter
 MO_GEWITTER	Gewitter
+RR_GEWITTER	Gewitter
 GLATTEIS	Glatteis
 JA_GLATTEIS	Glatteis
 MO_GLATTEIS	Glatteis
 GRAUPEL	Graupel
 JA_GRAUPEL	Graupel
 MO_GRAUPEL	Graupel
+RR_GRAUPEL	Graupel
 HAGEL	Hagel
 JA_HAGEL	Hagel
 MO_HAGEL	Hagel
+RR_HAGEL	Hagel
 P_STD	Luftdruck
 PM	Luftdruck
-PP_10	Luftdruck
 P	Luftdruck_NN
 P0	Luftdruck_Stationshoehe
 PP_TER	Luftdruck_Terminwert
@@ -195,30 +197,24 @@ JA_TT	Lufttemperatur
 MO_TT	Lufttemperatur
 TMK	Lufttemperatur
 TT	Lufttemperatur
-TT_10	Lufttemperatur
 TT_STD	Lufttemperatur
 TT_TU	Lufttemperatur
-TM5_10	Lufttemperatur_5cm
-TX5_10	Lufttemperatur_5cm_max
 TGK	Lufttemperatur_5cm_min
-TN5_10	Lufttemperatur_5cm_min
 JA_MX_TX	Lufttemperatur_AbsMax
 MX_TX	Lufttemperatur_AbsMax
 JA_MX_TN	Lufttemperatur_AbsMin
 MX_TN	Lufttemperatur_AbsMin
 JA_TX	Lufttemperatur_Max
 MO_TX	Lufttemperatur_Max
-TX_10	Lufttemperatur_Max
 TXK	Lufttemperatur_Max
 JA_TN	Lufttemperatur_Min
 MO_TN	Lufttemperatur_Min
-TN_10	Lufttemperatur_Min
 TNK	Lufttemperatur_Min
 TT_TER	Lufttemperatur_Terminwert
 JA_NEBEL	Nebel
 MO_NEBEL	Nebel
 NEBEL	Nebel
-RWS_DAU_10	Niederschlagsdauer
+RR_NEBEL	Nebel
 RSF	Niederschlagsform
 RSKF	Niederschlagsform
 WRTR	Niederschlagsform
@@ -226,18 +222,11 @@ JA_RR	Niederschlagshoehe
 MO_RR	Niederschlagshoehe
 R1	Niederschlagshoehe
 RS	Niederschlagshoehe
-RS_01	Niederschlagshoehe
 RSK	Niederschlagshoehe
-RWS_10	Niederschlagshoehe
 JA_MX_RS	Niederschlagshoehe_Max
 MX_RS	Niederschlagshoehe_Max
-RTH_01	Niederschlagshoehe_Tropfen
-RWH_01	Niederschlagshoehe_Wippe
 RS_IND	Niederschlagsindikator
-RS_IND_01	Niederschlagsindikator
-RWS_IND_10	Niederschlagsindikator
 REIF	Reif
-RF_10	Relative_Feuchte
 RF_STD	Relative_Feuchte
 RF_TU	Relative_Feuchte
 UPM	Relative_Feuchte
@@ -256,20 +245,16 @@ V_VV	Sichtweite
 VK_TER	Sichtweite
 JA_SD_S	Sonnenscheindauer
 MO_SD_S	Sonnenscheindauer
-SD_10	Sonnenscheindauer
 SD_LBERG	Sonnenscheindauer
 SD_SO	Sonnenscheindauer
 SD_STRAHL	Sonnenscheindauer
 SDK	Sonnenscheindauer
 ATMO_LBERG	Strahlung_Atmospaere
 ATMO_STRAHL	Strahlung_Atmospaere
-GS_10	Strahlung_Global
 FG_LBERG	Strahlung_Global_kurzwellig
 FG_STRAHL	Strahlung_Global_kurzwellig
-DS_10	Strahlung_Himmel_diffus
 FD_LBERG	Strahlung_Himmel_diffus
 FD_STRAHL	Strahlung_Himmel_diffus
-LS_10	Strahlung_langwellig
 JA_STURM_6	Sturm_6Bft
 MO_STURM_6	Sturm_6Bft
 STURM_6	Sturm_6Bft
@@ -280,22 +265,13 @@ JA_TAU	Tau
 MO_TAU	Tau
 TAU	Tau
 TD	Taupunkttemperatur
-TD_10	Taupunkttemperatur
 TD_STD	Taupunkttemperatur
 F	Windgeschwindigkeit
 FF	Windgeschwindigkeit
-FF_10	Windgeschwindigkeit
 FM	Windgeschwindigkeit
-FX_10	Windgeschwindigkeit_Max
-FMX_10	Windgeschwindigkeit_MaxMean
-FNX_10	Windgeschwindigkeit_Min
-SLA_10	Windgeschwindigkeit_STABW_lat
-SLO_10	Windgeschwindigkeit_STABW_lon
 D	Windrichtung
 DD	Windrichtung
-DD_10	Windrichtung
 DK_TER	Windrichtung
-DX_10	Windrichtung_Maxwind
 FX	Windspitze
 JA_MX_FX	Windspitze
 MX_FX	Windspitze
@@ -318,9 +294,31 @@ V_S1_HHS	Wolkenhoehe_Schicht1
 V_S2_HHS	Wolkenhoehe_Schicht2
 V_S3_HHS	Wolkenhoehe_Schicht3
 V_S4_HHS	Wolkenhoehe_Schicht4
-RR_GEWITTER	Gewitter
-RR_GRAUPEL	Graupel
-RR_HAGEL	Hagel
-RR_NEBEL	Nebel
+JA_EISTAGE	Anzahl_Eistage
+JA_FROSTTAGE	Anzahl_Frosttage
+JA_HEISSE_TAGE	Anzahl_Heisse_Tage
+JA_RR_GE_0_1_MM	Anzahl_Niederschlag00.1
+JA_RR_GE_1_0_MM	Anzahl_Niederschlag01
+JA_RR_GE_10_0_MM	Anzahl_Niederschlag10
+JA_RR_GE_2_5_MM	Anzahl_Niederschlag02.5
+JA_RR_GE_20_0_MM	Anzahl_Niederschlag20
+JA_RR_GE_5_0_MM	Anzahl_Niederschlag05
+JA_SH_GE_1_0_CM	Anzahl_Schnee1
+JA_SH_GE_5_0_CM	Anzahl_Schnee5
+JA_SOMMERTAGE	Anzahl_Sommertage
+JA_TROPENNAECHTE	Anzahl_Tropennaechte
+MO_EISTAGE	Anzahl_Eistage
+MO_FROSTTAGE	Anzahl_Frosttage
+MO_HEISSE_TAGE	Anzahl_Heisse_Tage
+MO_RR_GE_0_1_MM	Anzahl_Niederschlag00.1
+MO_RR_GE_1_0_MM	Anzahl_Niederschlag01
+MO_RR_GE_10_0_MM	Anzahl_Niederschlag10
+MO_RR_GE_2_5_MM	Anzahl_Niederschlag02.5
+MO_RR_GE_20_0_MM	Anzahl_Niederschlag20
+MO_RR_GE_5_0_MM	Anzahl_Niederschlag05
+MO_SH_GE_1_0_CM	Anzahl_Schnee1
+MO_SH_GE_5_0_CM	Anzahl_Schnee5
+MO_SOMMERTAGE	Anzahl_Sommertage
+MO_TROPENNAECHTE	Anzahl_Tropennaechte
 "))
 rownames(dwdparams) <- dwdparams$Parameter
