@@ -37,6 +37,9 @@
 #' # 477'236 (2022-04-28) # 5_minutes files added
 #' # 482'907 (2022-04-29)
 #' # 497'190 (2022-05-13)
+#' # 576'768 (2023-04-06) # lots of duplicate historical files in transition period
+#' # 577'160 (2023-04-12)
+#' # 571'667 (2023-04-14)
 #' 
 #' # gridbase
 #' #  49'247 (2019-05-26)
@@ -60,6 +63,9 @@
 #' #  35'874 (2022-04-28)
 #' #  35'937 (2022-04-29)
 #' #  36'630 (2022-05-13)
+#' #  36'059 (2023-04-06)
+#' #  36'356 (2023-04-12)
+#' #  36'454 (2023-04-14)
 #' 
 #' @param dwdlocal Read "DWDdata/INDEX_of_DWD_.txt" instead of calling
 #'                 [indexFTP()]? DEFAULT: FALSE
@@ -165,9 +171,10 @@ rv_df <- berryFunctions::sortDF(rv_df, "Par", decreasing=FALSE)
 rv_df <- berryFunctions::sortDF(rv_df, "Kurz", decreasing=FALSE)
 colnames(rv_df)[1] <- "Parameter"
 write.table(rv_df, "misc/params.txt", sep="\t", quote=FALSE, row.names=FALSE)
-message("- Copy content of 'misc/params.txt' to 'params.xlsx'.
+message("- Copy content of 'misc/params.txt' to 'params_input' at
+https://docs.google.com/spreadsheets/d/1qXQ1bSLW5TJnJgpUXIID3mVNYS6YZaHbsoe22LmBIAk/edit#gid=100501290
 - Manually add 'Kurz' entries.
-- Copy sheet 'output' to dwdparams in R/readVars.R")
+- Copy sheet 'params_output' to dwdparams in R/readVars.R")
 }
 #
 # check for duplicates:
